# -*- coding: utf-8 -*-

# Copyright 2024 University of Liège
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .builder import Builder
import openmdao.api as om
import numpy as np

class FakeStructMesh(om.IndepVarComp):
    """Initial structural surface mesh of lifting body
    """
    def initialize(self):
        self.options.declare('xs', desc='structural coordinates')

    def setup(self):
        self.add_output('x_struct0', val=self.options['xs'], desc='initial structural surface node coordinates')

class FakeStructSolver(om.ExplicitComponent):
    """OpenMDAO structural wrapper
    """
    def initialize(self):
        self.options.declare('xs', desc='structural coordinates')
        self.options.declare('qm', desc='modal displacements')
        self.options.declare('m', desc='mass matrix')
        self.options.declare('k', desc='stiffness matrix')

    def setup(self):
        self.add_input('x_struct0', val=self.options['xs'], desc='structural coordinates')
        self.add_output('q_struct', val=self.options['qm'], desc='modal displacements')
        self.add_output('M', val=self.options['m'], desc='mass matrix')
        self.add_output('K', val=self.options['k'], desc='stiffness matrix')

class FakeStructBuilder(Builder):
    """Structural modal solution of a wing (read from file)
    """
    def __init__(self, cfg):
        # Read modes
        modes = np.loadtxt(cfg['File'], delimiter=',', skiprows=1)
        nn = modes.shape[0]
        nm = (modes.shape[1] - 3) // 3
        # Store (flatten)
        self.__n_nodes = nn
        self.__x_struct = np.zeros(3 * nn)
        self.__q_modes = np.zeros((3 * nn, nm))
        for i in range(nn):
            for k in range(3):
                self.__x_struct[3 * i + k] = modes[i, k]
            for j in range(nm):
                for k in range(3):
                    self.__q_modes[3 * i + k, j] = modes[i, 3 + 3 * j + k]
        # Mass and stiffness matrices
        self.__m_mat = cfg['M']
        self.__k_mat = cfg['K']

    def get_mesh(self):
        """Return OpenMDAO component to get the initial surface mesh coordinates
        """
        return FakeStructMesh(xs=self.__x_struct)

    def get_solver(self):
        """Return OpenMDAO component containing the solver
        """
        return FakeStructSolver(xs=self.__x_struct, qm=self.__q_modes, m=self.__m_mat, k=self.__k_mat)

    def get_number_of_nodes(self):
        """Return the number of surface nodes
        """
        return self.__n_nodes
